@@ -1,6 +1,6 @@
 /* client.c
  *
- * $Id: client.c,v 1.86 2000/02/06 22:10:55 nisse Exp $ */
+ * $Id: client.c,v 1.89 2000/03/16 22:29:03 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -41,7 +41,6 @@
 #include "translate_signal.h"
 #include "tty.h"
 #include "unpad.h"
-#include "version.h"
 #include "werror.h"
 #include "xalloc.h"
 #include "compress.h"
@@ -357,8 +356,8 @@ do_receive(struct ssh_channel *c,
 
 /* We may send more data */
 static void
-do_send(struct ssh_channel *s,
-	struct ssh_connection *c UNUSED)
+do_send_adjust(struct ssh_channel *s,
+	       UINT32 i UNUSED)
 {
   CAST(client_session, self, s);
 
@@ -400,12 +399,12 @@ do_client_io(struct command *s UNUSED,
 					HANDLER_CONTEXT);
 
   /* Set up the fd we read from. */
-  channel->send = do_send;
+  channel->send_adjust = do_send_adjust;
 
   session->in->read = make_channel_read_data(channel);
 
   /* FIXME: Perhaps there is some way to arrange that channel.c calls
-   * the CHANNEL_SEND method instead? */
+   * the CHANNEL_SEND_ADJUST method instead? */
   if (session->super.send_window_size)
     session->in->want_read = 1;
   
@@ -522,3 +521,36 @@ do_format_shell_request(struct channel_request_command *s UNUSED,
 struct channel_request_command request_shell =
 { { STATIC_HEADER, do_channel_request_command }, do_format_shell_request };
 
+
+/* GABA:
+   (class
+     (name exec_request)
+     (super channel_request_command)
+     (vars
+       (command string)))
+*/
+
+static struct lsh_string *
+do_format_exec_request(struct channel_request_command *s,
+		       struct ssh_channel *channel,
+		       struct command_continuation **c)
+{
+  CAST(exec_request, self, s);
+
+  verbose("lsh: Requesting remote exec.\n");
+
+  return format_channel_request(ATOM_EXEC, channel,
+				!!*c, "%S", self->command);
+}
+
+struct command *
+make_exec_request(struct lsh_string *command)
+{
+  NEW(exec_request, req);
+
+  req->super.format_request = do_format_exec_request;
+  req->super.super.call = do_channel_request_command;
+  req->command = command;
+
+  return &req->super.super;
+}
