@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: client.h,v 1.27 2000/02/05 17:05:33 nisse Exp $ */
+ * $Id: client.h,v 1.28 2000/03/07 18:14:07 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -68,5 +68,7 @@ struct ssh_channel *make_client_session(struct lsh_fd *in,
 					UINT32 max_window,
 					int *exit_status);
 
+struct command *
+make_exec_request(struct lsh_string *command);
      
 #endif /* LSH_CLIENT_H_INCLUDED */
