@@ -2,7 +2,7 @@
  *
  * user authorization database
  *
- * $Id: server_authorization.c,v 1.7 1999/11/28 18:32:10 nisse Exp $ */
+ * $Id: server_authorization.c,v 1.9 2000/03/07 23:41:22 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -24,14 +24,14 @@
  */
 
 #include "server_authorization.h"
-#include "xalloc.h"
-#include "sexp.h"
-#include "spki.h"
+
 #include "format.h"
 #include "server_userauth.h"
+#include "sexp.h"
+#include "spki.h"
+#include "xalloc.h"
 
-#include <sys/stat.h>
-#include <unistd.h>
+#include <assert.h>
 
 #include "server_authorization.c.x"
 
@@ -51,24 +51,16 @@
 static struct verifier *
 do_key_lookup(struct lookup_verifier *c,
 	      int method,
-	      struct lsh_string *keyholder,
+	      struct user *keyholder,
 	      struct lsh_string *key)
 {
   CAST(authorization_db, closure, c);
   struct lsh_string *filename;
 
-  struct stat st;
-  struct unix_user *user;
   struct dsa_verifier *v;
 
-  /* FIXME: Hmm. User lookup is done twice, both here and higher up in
-   * server_publickey.c: do_authenticate. It might make more sense to
-   * pass this data instead of the keyholder username. */
+  assert(keyholder);
   
-  user = lookup_user(keyholder, 0);
-  if (!user)
-    return NULL;
-
   if (method != ATOM_SSH_DSS)
     return NULL;
 
@@ -82,8 +74,7 @@ do_key_lookup(struct lookup_verifier *c,
   
   /* FIXME: Proper spki acl reading should go here. */
   
-  filename = ssh_format("%lS/.lsh/%lS/%lxfS%c", 
-			user->home,
+  filename = ssh_format(".lsh/%lS/%lxfS%c", 
 			closure->index_name,
 			hash_string(closure->hashalgo,
 				    sexp_format(dsa_to_spki_public_key(&v->public),
@@ -91,11 +82,9 @@ do_key_lookup(struct lookup_verifier *c,
 				    1),				    
 			0);
 
-  if (stat(filename->data, &st) == 0)
-    {
-      lsh_string_free(filename);
-      return &v->super;
-    }
+  if (USER_FILE_EXISTS(keyholder, filename, 1))
+    return &v->super;
+
   lsh_string_free(filename);
   return NULL;
 }
