@@ -1,6 +1,6 @@
 /* server_session.h
  *
- * $Id: server_session.h,v 1.4 1999/11/29 21:32:50 nisse Exp $
+ * $Id: server_session.h,v 1.6 2000/03/14 21:05:01 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -33,24 +33,35 @@
 #include <assert.h>
 #include <string.h>
 
-struct ssh_channel *make_server_session(struct unix_user *user,
-					UINT32 max_window,
-					struct alist *request_types);
+struct ssh_channel *
+make_server_session(struct user *user,
+		    UINT32 max_window,
+		    struct alist *request_types);
 struct command *
 make_server_connection_service(struct alist *session_requests);
 
-struct channel_open *make_open_session(struct unix_user *user,
-				       struct alist *session_requests);
+struct channel_open *
+make_open_session(struct user *user,
+		  struct alist *session_requests);
 
-struct channel_request *make_shell_handler(struct io_backend *backend,
-					   struct reap *reap);
+struct channel_request *
+make_shell_handler(struct io_backend *backend,
+		   struct reap *reap);
 
-struct lsh_string *format_exit_signal(struct ssh_channel *channel,
-				      int core, int signal);
-struct lsh_string *format_exit(struct ssh_channel *channel, int value);
+struct channel_request *
+make_exec_handler(struct io_backend *backend,
+		  struct reap *reap);
 
-struct resource *make_process_resource(pid_t pid, int signal);
+struct lsh_string *
+format_exit_signal(struct ssh_channel *channel,
+		   int core, int signal);
+struct lsh_string *
+format_exit(struct ssh_channel *channel, int value);
 
-struct channel_request *make_pty_handler(void);
+struct resource *
+make_process_resource(pid_t pid, int signal);
+
+struct channel_request *
+make_pty_handler(void);
 
 #endif /* LSH_SERVER_SESSION_H_INCLUDED */
