@@ -1,6 +1,6 @@
 /* tcpforward_commands.c
  *
- * $Id: tcpforward_commands.c,v 1.21 2000/02/20 18:59:41 nisse Exp $
+ * $Id: tcpforward_commands.c,v 1.22 2000/03/16 22:47:13 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -74,7 +74,7 @@ static struct catch_report_collect catch_channel_open
  * the party receiving a open-tcp request, when a channel to the
  * target has been opened. */
 
-#define TCPIP_WINDOW_SIZE (SSH_MAX_PACKET << 3)
+#define TCPIP_WINDOW_SIZE 10000
 
 /* NOTE: This command does not do any remembering. */
 static void
